@@ -56,9 +56,10 @@ import pprint
 import copy
 
 import django.db
-from django.contrib.auth.models import User, Group, SiteProfileNotAvailable
+from django.contrib.auth.models import User, Group, Permission, SiteProfileNotAvailable
 from django.core.cache import cache
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
+import django.dispatch
 
 from django_auth_ldap.config import _LDAPConfig, LDAPSearch, LDAPGroupType
 
@@ -66,11 +67,19 @@ from django_auth_ldap.config import _LDAPConfig, LDAPSearch, LDAPGroupType
 logger = _LDAPConfig.get_logger()
 
 
+# Signals for populating user objects.
+populate_user = django.dispatch.Signal(providing_args=["user", "ldap_user"])
+populate_user_profile = django.dispatch.Signal(providing_args=["profile", "ldap_user"])
+
+
 class LDAPBackend(object):
     """
     The main backend class. This implements the auth backend API, although it
     actually delegates most of its work to _LDAPUser, which is defined next.
     """
+    supports_anonymous_user = False
+    supports_object_permissions = False
+
     ldap = None # The cached ldap module (or mock object)
     
     def __init__(self):
@@ -155,7 +164,7 @@ class LDAPBackend(object):
         username is the Django-friendly username of the user. ldap_user.dn is
         the user's DN and ldap_user.attrs contains all of their LDAP attributes.
         """
-        return User.objects.get_or_create(username=username)
+        return User.objects.get_or_create(username__iexact=username, defaults={'username': username.lower()})
 
     def ldap_to_django_username(self, username):
         return username
@@ -416,27 +425,39 @@ class _LDAPUser(object):
         
         username = self.backend.ldap_to_django_username(self._username)
 
-        (self._user, created) = self.backend.get_or_create_user(username, self)
+        self._user, created = self.backend.get_or_create_user(username, self)
+        self._user.ldap_user = self
+        self._user.ldap_username = self._username
+
+        should_populate = force_populate or ldap_settings.AUTH_LDAP_ALWAYS_UPDATE_USER or created
 
         if created:
             logger.debug("Created Django user %s", username)
             self._user.set_unusable_password()
             save_user = True
 
-        if(force_populate or ldap_settings.AUTH_LDAP_ALWAYS_UPDATE_USER or created):
+        if should_populate:
             logger.debug("Populating Django user %s", username)
             self._populate_user()
-            self._populate_and_save_user_profile()
             save_user = True
 
         if ldap_settings.AUTH_LDAP_MIRROR_GROUPS:
             self._mirror_groups()
 
+        # Give the client a chance to finish populating the user just before
+        # saving.
+        if should_populate:
+            signal_responses = populate_user.send(self.backend.__class__, user=self._user, ldap_user=self)
+            if len(signal_responses) > 0:
+                save_user = True
+
         if save_user:
             self._user.save()
 
-        self._user.ldap_user = self
-        self._user.ldap_username = self._username
+        # We populate the profile after the user model is saved to give the
+        # client a chance to create the profile.
+        if should_populate:
+            self._populate_and_save_user_profile()
 
     def _populate_user(self):
         """
@@ -450,7 +471,7 @@ class _LDAPUser(object):
             try:
                 setattr(self._user, field, self.attrs[attr][0])
             except (KeyError, IndexError):
-                pass
+                logger.warning("%s does not have a value for the attribute %s", self.dn, attr)
     
     def _populate_user_from_group_memberships(self):
         for field, group_dn in ldap_settings.AUTH_LDAP_USER_FLAGS_BY_GROUP.iteritems():
@@ -463,19 +484,27 @@ class _LDAPUser(object):
         """
         try:
             profile = self._user.get_profile()
+            save_profile = False
+
+            logger.debug("Populating Django user profile for %s", self._user.username)
 
             for field, attr in ldap_settings.AUTH_LDAP_PROFILE_ATTR_MAP.iteritems():
                 try:
                     # user_attrs is a hash of lists of attribute values
                     setattr(profile, field, self.attrs[attr][0])
+                    save_profile = True
                 except (KeyError, IndexError):
-                    pass
+                    logger.warning("%s does not have a value for the attribute %s", self.dn, attr)
 
-            if len(ldap_settings.AUTH_LDAP_PROFILE_ATTR_MAP) > 0:
+            signal_responses = populate_user_profile.send(self.backend.__class__, profile=profile, ldap_user=self)
+            if len(signal_responses) > 0:
+                save_profile = True
+
+            if save_profile:
                 profile.save()
         except (SiteProfileNotAvailable, ObjectDoesNotExist):
-            pass
-    
+            logger.debug("Django user %s does not have a profile to populate", self._user.username)
+
     def _mirror_groups(self):
         """
         Mirrors the user's LDAP groups in the Django database and updates the
@@ -495,40 +524,14 @@ class _LDAPUser(object):
         """
         Populates self._group_permissions based on LDAP group membership and
         Django group permissions.
-        
-        The SQL is lifted from ModelBackend, with modifications.
         """
         group_names = self._get_groups().get_group_names()
-        placeholders = ', '.join(['%s'] * len(group_names))
-        
-        cursor = django.db.connection.cursor()
-        # The SQL below works out to the following, after DB quoting:
-        # cursor.execute("""
-        #     SELECT ct."app_label", p."codename"
-        #     FROM "auth_permission" p, "auth_group_permissions" gp, "auth_group" g, "django_content_type" ct
-        #     WHERE p."id" = gp."permission_id"
-        #         AND gp."group_id" = g."id"
-        #         AND ct."id" = p."content_type_id"
-        #         AND g."name" IN (%s, %s, ...)""", ['group1', 'group2', ...])
-        qn = django.db.connection.ops.quote_name
-        sql = u"""
-            SELECT ct.%s, p.%s
-            FROM %s p, %s gp, %s g, %s ct
-            WHERE p.%s = gp.%s
-                AND gp.%s = g.%s
-                AND ct.%s = p.%s
-                AND g.%s IN (%s)""" % (
-            qn('app_label'), qn('codename'),
-            qn('auth_permission'), qn('auth_group_permissions'),
-            qn('auth_group'), qn('django_content_type'),
-            qn('id'), qn('permission_id'),
-            qn('group_id'), qn('id'),
-            qn('id'), qn('content_type_id'),
-            qn('name'), placeholders)
-        
-        cursor.execute(sql, group_names)
-        self._group_permissions = \
-            set([u"%s.%s" % (row[0], row[1]) for row in cursor.fetchall()])
+
+        perms = Permission.objects.filter(group__name__in=group_names
+            ).values_list('content_type__app_label', 'codename'
+            ).order_by()
+
+        self._group_permissions = set(["%s.%s" % (ct, name) for ct, name in perms])
 
     def _get_groups(self):
         """
