@@ -32,9 +32,12 @@ except NameError:
 import sys
 import logging
 
+from django.conf import settings
+import django.db.models.signals
 from django.contrib.auth.models import User, Permission, Group
 from django.test import TestCase
 
+import django_auth_ldap.models
 from django_auth_ldap import backend
 from django_auth_ldap.config import _LDAPConfig, LDAPSearch
 from django_auth_ldap.config import PosixGroupType, MemberDNGroupType, NestedMemberDNGroupType
@@ -237,7 +240,7 @@ class MockLDAP(object):
         
         if(who == '' and cred == ''):
             success = True 
-        elif self._compare_s(who, 'userPassword', cred):
+        elif self._compare_s(who.lower(), 'userPassword', cred):
             success = True
 
         if success:
@@ -455,6 +458,20 @@ class LDAPTest(TestCase):
         self.assertEqual(self.mock_ldap.ldap_methods_called(),
             ['initialize', 'simple_bind_s'])
 
+    def test_new_user_lowercase(self):
+        self._init_settings(
+            AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test'
+        )
+        user_count = User.objects.count()
+        
+        user = self.backend.authenticate(username='Alice', password='password')
+        
+        self.assert_(not user.has_usable_password())
+        self.assertEqual(user.username, 'alice')
+        self.assertEqual(User.objects.count(), user_count + 1)
+        self.assertEqual(self.mock_ldap.ldap_methods_called(),
+            ['initialize', 'simple_bind_s'])
+
     def test_simple_bind_bad_user(self):
         self._init_settings(
             AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test'
@@ -494,6 +511,22 @@ class LDAPTest(TestCase):
         self.assert_(user is not None)
         self.assertEqual(User.objects.count(), user_count)
 
+    def test_existing_user_insensitive(self):
+        self._init_settings(
+            AUTH_LDAP_USER_SEARCH=LDAPSearch(
+                "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
+                )
+            )
+        self.mock_ldap.set_return_value('search_s',
+            ("ou=people,o=test", 2, "(uid=Alice)", None, 0), [self.alice])
+        User.objects.create(username='alice')
+        
+        user = self.backend.authenticate(username='Alice', password='password')
+
+        self.assert_(user is not None)
+        self.assertEqual(user.username, 'alice')
+        self.assertEqual(User.objects.count(), 1)
+
     def test_convert_username(self):
         class MyBackend(backend.LDAPBackend):
             def ldap_to_django_username(self, username):
@@ -626,7 +659,7 @@ class LDAPTest(TestCase):
         self.assert_(user is not None)
         self.assertEqual(user.username, u'dreßler')
         self.assertEqual(user.last_name, u'Dreßler')
-    
+
     def test_populate_user(self):
         self._init_settings(
             AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -639,6 +672,41 @@ class LDAPTest(TestCase):
         self.assertEqual(user.first_name, 'Alice')
         self.assertEqual(user.last_name, 'Adams')
 
+    def test_signal_populate_user(self):
+        self._init_settings(
+            AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test'
+        )
+        def handle_populate_user(sender, **kwargs):
+            self.assert_('user' in kwargs and 'ldap_user' in kwargs)
+            kwargs['user'].populate_user_handled = True
+        backend.populate_user.connect(handle_populate_user)
+
+        user = self.backend.authenticate(username='alice', password='password')
+
+        self.assert_(user.populate_user_handled)
+
+    def test_signal_populate_user_profile(self):
+        settings.AUTH_PROFILE_MODULE = 'django_auth_ldap.TestProfile'
+
+        self._init_settings(
+            AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test'
+        )
+
+        def handle_user_saved(sender, **kwargs):
+            if kwargs['created']:
+                django_auth_ldap.models.TestProfile.objects.create(user=kwargs['instance'])
+
+        def handle_populate_user_profile(sender, **kwargs):
+            self.assert_('profile' in kwargs and 'ldap_user' in kwargs)
+            kwargs['profile'].populated = True
+
+        django.db.models.signals.post_save.connect(handle_user_saved, sender=User)
+        backend.populate_user_profile.connect(handle_populate_user_profile)
+
+        user = self.backend.authenticate(username='alice', password='password')
+
+        self.assert_(user.get_profile().populated)
+
     def test_no_update_existing(self):
         self._init_settings(
             AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -669,7 +737,7 @@ class LDAPTest(TestCase):
         self.assert_(alice is not None)
         self.assert_(bob is None)
         self.assertEqual(self.mock_ldap.ldap_methods_called(),
-            ['initialize', 'simple_bind_s', 'compare_s', 'initialize', 'simple_bind_s', 'compare_s'])
+            ['initialize', 'simple_bind_s', 'simple_bind_s', 'compare_s', 'initialize', 'simple_bind_s', 'simple_bind_s', 'compare_s'])
 
     def test_dn_group_membership(self):
         self._init_settings(
@@ -784,12 +852,33 @@ class LDAPTest(TestCase):
         
         alice = User.objects.create(username='alice')
         alice = self.backend.get_user(alice.pk)
-        
+
         self.assertEqual(self.backend.get_group_permissions(alice), set(["auth.add_user", "auth.change_user"]))
         self.assertEqual(self.backend.get_all_permissions(alice), set(["auth.add_user", "auth.change_user"]))
         self.assert_(self.backend.has_perm(alice, "auth.add_user"))
         self.assert_(self.backend.has_module_perms(alice, "auth"))
-    
+
+    def test_empty_group_permissions(self):
+        self._init_settings(
+            AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
+            AUTH_LDAP_GROUP_SEARCH=LDAPSearch('ou=groups,o=test', self.mock_ldap.SCOPE_SUBTREE),
+            AUTH_LDAP_GROUP_TYPE=MemberDNGroupType(member_attr='member'),
+            AUTH_LDAP_FIND_GROUP_PERMS=True
+        )
+        self._init_groups()
+        self.mock_ldap.set_return_value('search_s',
+            ("ou=groups,o=test", 2, "(&(objectClass=*)(member=uid=bob,ou=people,o=test))", None, 0),
+            []
+        )
+        
+        bob = User.objects.create(username='bob')
+        bob = self.backend.get_user(bob.pk)
+        
+        self.assertEqual(self.backend.get_group_permissions(bob), set())
+        self.assertEqual(self.backend.get_all_permissions(bob), set())
+        self.assert_(not self.backend.has_perm(bob, "auth.add_user"))
+        self.assert_(not self.backend.has_module_perms(bob, "auth"))
+
     def test_posix_group_permissions(self):
         self._init_settings(
             AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -988,7 +1077,7 @@ class LDAPTest(TestCase):
         self._init_settings(
             AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
             AUTH_LDAP_START_TLS=False,
-            )
+        )
 
         self.assert_(not self.mock_ldap.tls_enabled)
         self.backend.authenticate(username='alice', password='password')
@@ -998,12 +1087,29 @@ class LDAPTest(TestCase):
         self._init_settings(
             AUTH_LDAP_USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
             AUTH_LDAP_START_TLS=True,
-            )
+        )
 
         self.assert_(not self.mock_ldap.tls_enabled)
         self.backend.authenticate(username='alice', password='password')
         self.assert_(self.mock_ldap.tls_enabled)
 
+    def test_null_search_results(self):
+        """
+        Reportedly, some servers under some circumstances can return search
+        results of the form (None, '<some ldap url>'). We're not sure what they
+        are, but we filter those out so we don't trip over them.
+        """
+        self._init_settings(
+            AUTH_LDAP_USER_SEARCH=LDAPSearch(
+                "ou=people,o=test", self.mock_ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
+                )
+            )
+        self.mock_ldap.set_return_value('search_s',
+            ("ou=people,o=test", 2, "(uid=alice)", None, 0), [self.alice, (None, '')])
+
+        self.backend.authenticate(username='alice', password='password')
+
+
     def _init_settings(self, **kwargs):
         backend.ldap_settings = TestSettings(**kwargs)
     
